@@ -1,11 +1,12 @@
 package org.apache.lucene.analysis;
 
 /**
- * Copyright 2004 The Apache Software Foundation
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -27,10 +28,6 @@ public abstract class TokenFilter extends TokenStream {
   /** The source of tokens for this filter. */
   protected TokenStream input;
 
-  /** Call TokenFilter(TokenStream) instead.
-   * @deprecated */
-  protected TokenFilter() {}
-
   /** Construct a token stream filtering the given input. */
   protected TokenFilter(TokenStream input) {
     this.input = input;
