@@ -1,11 +1,12 @@
 package org.apache.lucene.search;
 
 /**
- * Copyright 2004 The Apache Software Foundation
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -27,7 +28,11 @@ import org.apache.lucene.index.Term;
 /** Implements search over a single IndexReader.
  *
  * <p>Applications usually need only call the inherited {@link #search(Query)}
- * or {@link #search(Query,Filter)} methods.
+ * or {@link #search(Query,Filter)} methods. For performance reasons it is 
+ * recommended to open only one IndexSearcher and use it for all of your searches.
+ * 
+ * <p>Note that you can only access Hits from an IndexSearcher as long as it is
+ * not yet closed, otherwise an IOException will be thrown. 
  */
 public class IndexSearcher extends Searcher {
   IndexReader reader;
@@ -53,6 +58,11 @@ public class IndexSearcher extends Searcher {
     this.closeReader = closeReader;
   }
 
+  /** Return the {@link IndexReader} this searches. */
+  public IndexReader getIndexReader() {
+    return reader;
+  }
+
   /**
    * Note that the underlying IndexReader is not closed, if
    * IndexSearcher was constructed with IndexSearcher(IndexReader r).
@@ -80,82 +90,44 @@ public class IndexSearcher extends Searcher {
   }
 
   // inherit javadoc
-  public TopDocs search(Query query, Filter filter, final int nDocs)
+  public TopDocs search(Weight weight, Filter filter, final int nDocs)
        throws IOException {
-    Scorer scorer = query.weight(this).scorer(reader);
-    if (scorer == null)
-      return new TopDocs(0, new ScoreDoc[0]);
 
-    final BitSet bits = filter != null ? filter.bits(reader) : null;
-    final HitQueue hq = new HitQueue(nDocs);
-    final int[] totalHits = new int[1];
-    scorer.score(new HitCollector() {
-        private float minScore = 0.0f;
-	public final void collect(int doc, float score) {
-	  if (score > 0.0f &&			  // ignore zeroed buckets
-	      (bits==null || bits.get(doc))) {	  // skip docs not in bits
-	    totalHits[0]++;
-            if (hq.size() < nDocs || score >= minScore) {
-              hq.insert(new ScoreDoc(doc, score));
-              minScore = ((ScoreDoc)hq.top()).score; // maintain minScore
-            }
-	  }
-	}
-      });
+    if (nDocs <= 0)  // null might be returned from hq.top() below.
+      throw new IllegalArgumentException("nDocs must be > 0");
 
-    ScoreDoc[] scoreDocs = new ScoreDoc[hq.size()];
-    for (int i = hq.size()-1; i >= 0; i--)	  // put docs in array
-      scoreDocs[i] = (ScoreDoc)hq.pop();
-
-    return new TopDocs(totalHits[0], scoreDocs);
+    TopDocCollector collector = new TopDocCollector(nDocs);
+    search(weight, filter, collector);
+    return collector.topDocs();
   }
 
   // inherit javadoc
-  public TopFieldDocs search(Query query, Filter filter, final int nDocs,
+  public TopFieldDocs search(Weight weight, Filter filter, final int nDocs,
                              Sort sort)
-    throws IOException {
-    Scorer scorer = query.weight(this).scorer(reader);
-    if (scorer == null)
-      return new TopFieldDocs(0, new ScoreDoc[0], sort.fields);
+      throws IOException {
 
-    final BitSet bits = filter != null ? filter.bits(reader) : null;
-    final FieldSortedHitQueue hq =
-      new FieldSortedHitQueue(reader, sort.fields, nDocs);
-    final int[] totalHits = new int[1];
-    scorer.score(new HitCollector() {
-        public final void collect(int doc, float score) {
-          if (score > 0.0f &&			  // ignore zeroed buckets
-              (bits==null || bits.get(doc))) {	  // skip docs not in bits
-            totalHits[0]++;
-            hq.insert(new FieldDoc(doc, score));
-          }
-        }
-      });
-
-    ScoreDoc[] scoreDocs = new ScoreDoc[hq.size()];
-    for (int i = hq.size()-1; i >= 0; i--)	  // put docs in array
-      scoreDocs[i] = hq.fillFields ((FieldDoc) hq.pop());
-
-    return new TopFieldDocs(totalHits[0], scoreDocs, hq.getFields());
+    TopFieldDocCollector collector =
+      new TopFieldDocCollector(reader, sort, nDocs);
+    search(weight, filter, collector);
+    return (TopFieldDocs)collector.topDocs();
   }
 
-
   // inherit javadoc
-  public void search(Query query, Filter filter,
+  public void search(Weight weight, Filter filter,
                      final HitCollector results) throws IOException {
     HitCollector collector = results;
     if (filter != null) {
       final BitSet bits = filter.bits(reader);
       collector = new HitCollector() {
-	  public final void collect(int doc, float score) {
-	    if (bits.get(doc)) {		  // skip docs not in bits
-	      results.collect(doc, score);
-	    }
-	  }
-	};
+          public final void collect(int doc, float score) {
+            if (bits.get(doc)) {                  // skip docs not in bits
+              results.collect(doc, score);
+            }
+          }
+        };
     }
 
-    Scorer scorer = query.weight(this).scorer(reader);
+    Scorer scorer = weight.scorer(reader);
     if (scorer == null)
       return;
     scorer.score(collector);
@@ -170,8 +142,7 @@ public class IndexSearcher extends Searcher {
     return query;
   }
 
-  public Explanation explain(Query query, int doc) throws IOException {
-    return query.weight(this).explain(reader, doc);
+  public Explanation explain(Weight weight, int doc) throws IOException {
+    return weight.explain(reader, doc);
   }
-
 }
