@@ -1,11 +1,12 @@
 package org.apache.lucene.index;
 
 /**
- * Copyright 2004 The Apache Software Foundation
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+ * Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -22,17 +23,17 @@ import java.io.File;
 import junit.framework.TestCase;
 import junit.framework.TestSuite;
 import junit.textui.TestRunner;
-import org.apache.lucene.store.OutputStream;
+import org.apache.lucene.store.IndexOutput;
 import org.apache.lucene.store.Directory;
-import org.apache.lucene.store.InputStream;
+import org.apache.lucene.store.IndexInput;
 import org.apache.lucene.store.FSDirectory;
-import org.apache.lucene.store.RAMDirectory;
 import org.apache.lucene.store._TestHelper;
+import org.apache.lucene.util._TestUtil;
 
 
 /**
  * @author dmitrys@earthlink.net
- * @version $Id: TestCompoundFile.java,v 1.5 2004/03/29 22:48:06 cutting Exp $
+ * @version $Id: TestCompoundFile.java 497612 2007-01-18 22:47:03Z mikemccand $
  */
 public class TestCompoundFile extends TestCase
 {
@@ -58,8 +59,9 @@ public class TestCompoundFile extends TestCase
 
 
     public void setUp() throws IOException {
-        //dir = new RAMDirectory();
-        dir = FSDirectory.getDirectory(new File(System.getProperty("tempDir"), "testIndex"), true);
+       File file = new File(System.getProperty("tempDir"), "testIndex");
+       _TestUtil.rmDir(file);
+       dir = FSDirectory.getDirectory(file);
     }
 
 
@@ -67,7 +69,7 @@ public class TestCompoundFile extends TestCase
     private void createRandomFile(Directory dir, String name, int size)
     throws IOException
     {
-        OutputStream os = dir.createFile(name);
+        IndexOutput os = dir.createOutput(name);
         for (int i=0; i<size; i++) {
             byte b = (byte) (Math.random() * 256);
             os.writeByte(b);
@@ -85,7 +87,7 @@ public class TestCompoundFile extends TestCase
                                     int size)
     throws IOException
     {
-        OutputStream os = dir.createFile(name);
+        IndexOutput os = dir.createOutput(name);
         for (int i=0; i < size; i++) {
             os.writeByte(start);
             start ++;
@@ -95,8 +97,8 @@ public class TestCompoundFile extends TestCase
 
 
     private void assertSameStreams(String msg,
-                                   InputStream expected,
-                                   InputStream test)
+                                   IndexInput expected,
+                                   IndexInput test)
     throws IOException
     {
         assertNotNull(msg + " null expected", expected);
@@ -121,8 +123,8 @@ public class TestCompoundFile extends TestCase
 
 
     private void assertSameStreams(String msg,
-                                   InputStream expected,
-                                   InputStream actual,
+                                   IndexInput expected,
+                                   IndexInput actual,
                                    long seekTo)
     throws IOException
     {
@@ -137,8 +139,8 @@ public class TestCompoundFile extends TestCase
 
 
     private void assertSameSeekBehavior(String msg,
-                                        InputStream expected,
-                                        InputStream actual)
+                                        IndexInput expected,
+                                        IndexInput actual)
     throws IOException
     {
         // seek to 0
@@ -200,8 +202,8 @@ public class TestCompoundFile extends TestCase
             csw.close();
 
             CompoundFileReader csr = new CompoundFileReader(dir, name + ".cfs");
-            InputStream expected = dir.openFile(name);
-            InputStream actual = csr.openFile(name);
+            IndexInput expected = dir.openInput(name);
+            IndexInput actual = csr.openInput(name);
             assertSameStreams(name, expected, actual);
             assertSameSeekBehavior(name, expected, actual);
             expected.close();
@@ -224,15 +226,15 @@ public class TestCompoundFile extends TestCase
         csw.close();
 
         CompoundFileReader csr = new CompoundFileReader(dir, "d.csf");
-        InputStream expected = dir.openFile("d1");
-        InputStream actual = csr.openFile("d1");
+        IndexInput expected = dir.openInput("d1");
+        IndexInput actual = csr.openInput("d1");
         assertSameStreams("d1", expected, actual);
         assertSameSeekBehavior("d1", expected, actual);
         expected.close();
         actual.close();
 
-        expected = dir.openFile("d2");
-        actual = csr.openFile("d2");
+        expected = dir.openInput("d2");
+        actual = csr.openInput("d2");
         assertSameStreams("d2", expected, actual);
         assertSameSeekBehavior("d2", expected, actual);
         expected.close();
@@ -280,8 +282,8 @@ public class TestCompoundFile extends TestCase
 
         CompoundFileReader csr = new CompoundFileReader(dir, "test.cfs");
         for (int i=0; i<data.length; i++) {
-            InputStream check = dir.openFile(segment + data[i]);
-            InputStream test = csr.openFile(segment + data[i]);
+            IndexInput check = dir.openInput(segment + data[i]);
+            IndexInput test = csr.openInput(segment + data[i]);
             assertSameStreams(data[i], check, test);
             assertSameSeekBehavior(data[i], check, test);
             test.close();
@@ -307,22 +309,22 @@ public class TestCompoundFile extends TestCase
 
 
     public void testReadAfterClose() throws IOException {
-        demo_FSInputStreamBug((FSDirectory) dir, "test");
+        demo_FSIndexInputBug((FSDirectory) dir, "test");
     }
 
-    private void demo_FSInputStreamBug(FSDirectory fsdir, String file)
+    private void demo_FSIndexInputBug(FSDirectory fsdir, String file)
     throws IOException
     {
         // Setup the test file - we need more than 1024 bytes
-        OutputStream os = fsdir.createFile(file);
+        IndexOutput os = fsdir.createOutput(file);
         for(int i=0; i<2000; i++) {
             os.writeByte((byte) i);
         }
         os.close();
 
-        InputStream in = fsdir.openFile(file);
+        IndexInput in = fsdir.openInput(file);
 
-        // This read primes the buffer in InputStream
+        // This read primes the buffer in IndexInput
         byte b = in.readByte();
 
         // Close the file
@@ -339,21 +341,23 @@ public class TestCompoundFile extends TestCase
             // OK: this call correctly fails. We are now past the 1024 internal
             // buffer, so an actual IO is attempted, which fails
             b = in.readByte();
+            fail("expected readByte() to throw exception");
         } catch (IOException e) {
+          // expected exception
         }
     }
 
 
-    static boolean isCSInputStream(InputStream is) {
-        return is instanceof CompoundFileReader.CSInputStream;
+    static boolean isCSIndexInput(IndexInput is) {
+        return is instanceof CompoundFileReader.CSIndexInput;
     }
 
-    static boolean isCSInputStreamOpen(InputStream is) throws IOException {
-        if (isCSInputStream(is)) {
-            CompoundFileReader.CSInputStream cis =
-            (CompoundFileReader.CSInputStream) is;
+    static boolean isCSIndexInputOpen(IndexInput is) throws IOException {
+        if (isCSIndexInput(is)) {
+            CompoundFileReader.CSIndexInput cis =
+            (CompoundFileReader.CSIndexInput) is;
 
-            return _TestHelper.isFSInputStreamOpen(cis.base);
+            return _TestHelper.isFSIndexInputOpen(cis.base);
         } else {
             return false;
         }
@@ -365,47 +369,52 @@ public class TestCompoundFile extends TestCase
         CompoundFileReader cr = new CompoundFileReader(dir, "f.comp");
 
         // basic clone
-        InputStream expected = dir.openFile("f11");
-        assertTrue(_TestHelper.isFSInputStreamOpen(expected));
+        IndexInput expected = dir.openInput("f11");
 
-        InputStream one = cr.openFile("f11");
-        assertTrue(isCSInputStreamOpen(one));
+        // this test only works for FSIndexInput
+        if (_TestHelper.isFSIndexInput(expected)) {
 
-        InputStream two = (InputStream) one.clone();
-        assertTrue(isCSInputStreamOpen(two));
+          assertTrue(_TestHelper.isFSIndexInputOpen(expected));
 
-        assertSameStreams("basic clone one", expected, one);
-        expected.seek(0);
-        assertSameStreams("basic clone two", expected, two);
+          IndexInput one = cr.openInput("f11");
+          assertTrue(isCSIndexInputOpen(one));
 
-        // Now close the first stream
-        one.close();
-        assertTrue("Only close when cr is closed", isCSInputStreamOpen(one));
+          IndexInput two = (IndexInput) one.clone();
+          assertTrue(isCSIndexInputOpen(two));
 
-        // The following should really fail since we couldn't expect to
-        // access a file once close has been called on it (regardless of
-        // buffering and/or clone magic)
-        expected.seek(0);
-        two.seek(0);
-        assertSameStreams("basic clone two/2", expected, two);
+          assertSameStreams("basic clone one", expected, one);
+          expected.seek(0);
+          assertSameStreams("basic clone two", expected, two);
+
+          // Now close the first stream
+          one.close();
+          assertTrue("Only close when cr is closed", isCSIndexInputOpen(one));
+
+          // The following should really fail since we couldn't expect to
+          // access a file once close has been called on it (regardless of
+          // buffering and/or clone magic)
+          expected.seek(0);
+          two.seek(0);
+          assertSameStreams("basic clone two/2", expected, two);
 
 
-        // Now close the compound reader
-        cr.close();
-        assertFalse("Now closed one", isCSInputStreamOpen(one));
-        assertFalse("Now closed two", isCSInputStreamOpen(two));
+          // Now close the compound reader
+          cr.close();
+          assertFalse("Now closed one", isCSIndexInputOpen(one));
+          assertFalse("Now closed two", isCSIndexInputOpen(two));
 
-        // The following may also fail since the compound stream is closed
-        expected.seek(0);
-        two.seek(0);
-        //assertSameStreams("basic clone two/3", expected, two);
+          // The following may also fail since the compound stream is closed
+          expected.seek(0);
+          two.seek(0);
+          //assertSameStreams("basic clone two/3", expected, two);
 
 
-        // Now close the second clone
-        two.close();
-        expected.seek(0);
-        two.seek(0);
-        //assertSameStreams("basic clone two/4", expected, two);
+          // Now close the second clone
+          two.close();
+          expected.seek(0);
+          two.seek(0);
+          //assertSameStreams("basic clone two/4", expected, two);
+        }
 
         expected.close();
     }
@@ -419,11 +428,11 @@ public class TestCompoundFile extends TestCase
         CompoundFileReader cr = new CompoundFileReader(dir, "f.comp");
 
         // Open two files
-        InputStream e1 = dir.openFile("f11");
-        InputStream e2 = dir.openFile("f3");
+        IndexInput e1 = dir.openInput("f11");
+        IndexInput e2 = dir.openInput("f3");
 
-        InputStream a1 = cr.openFile("f11");
-        InputStream a2 = dir.openFile("f3");
+        IndexInput a1 = cr.openInput("f11");
+        IndexInput a2 = dir.openInput("f3");
 
         // Seek the first pair
         e1.seek(100);
@@ -498,11 +507,11 @@ public class TestCompoundFile extends TestCase
         CompoundFileReader cr = new CompoundFileReader(dir, "f.comp");
 
         // Open two files
-        InputStream e1 = cr.openFile("f11");
-        InputStream e2 = cr.openFile("f3");
+        IndexInput e1 = cr.openInput("f11");
+        IndexInput e2 = cr.openInput("f3");
 
-        InputStream a1 = (InputStream) e1.clone();
-        InputStream a2 = (InputStream) e2.clone();
+        IndexInput a1 = (IndexInput) e1.clone();
+        IndexInput a2 = (IndexInput) e2.clone();
 
         // Seek the first pair
         e1.seek(100);
@@ -576,7 +585,7 @@ public class TestCompoundFile extends TestCase
 
         // Open two files
         try {
-            InputStream e1 = cr.openFile("bogus");
+            IndexInput e1 = cr.openInput("bogus");
             fail("File not found");
 
         } catch (IOException e) {
@@ -591,7 +600,7 @@ public class TestCompoundFile extends TestCase
     public void testReadPastEOF() throws IOException {
         setUp_2();
         CompoundFileReader cr = new CompoundFileReader(dir, "f.comp");
-        InputStream is = cr.openFile("f2");
+        IndexInput is = cr.openInput("f2");
         is.seek(is.length() - 10);
         byte b[] = new byte[100];
         is.readBytes(b, 0, 10);
@@ -616,4 +625,26 @@ public class TestCompoundFile extends TestCase
         is.close();
         cr.close();
     }
+
+    /** This test that writes larger than the size of the buffer output
+     * will correctly increment the file pointer.
+     */
+    public void testLargeWrites() throws IOException {
+        IndexOutput os = dir.createOutput("testBufferStart.txt");
+
+        byte[] largeBuf = new byte[2048];
+        for (int i=0; i<largeBuf.length; i++) {
+            largeBuf[i] = (byte) (Math.random() * 256);
+        }
+
+        long currentPos = os.getFilePointer();
+        os.writeBytes(largeBuf, largeBuf.length);
+
+        try {
+            assertEquals(currentPos + largeBuf.length, os.getFilePointer());
+        } finally {
+            os.close();
+        }
+
+    }
 }
